# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .terminal_action_query_filter import TerminalActionQueryFilter
import pydantic
from .terminal_action_query_sort import TerminalActionQuerySort
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TerminalActionQuery(UncheckedBaseModel):
    filter: typing.Optional[TerminalActionQueryFilter] = pydantic.Field(default=None)
    """
    Options for filtering returned `TerminalAction`s
    """

    sort: typing.Optional[TerminalActionQuerySort] = pydantic.Field(default=None)
    """
    Option for sorting returned `TerminalAction` objects.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .catalog_custom_attribute_definition_selection_config_custom_attribute_selection import (
    CatalogCustomAttributeDefinitionSelectionConfigCustomAttributeSelectionParams,
)


class CatalogCustomAttributeDefinitionSelectionConfigParams(typing_extensions.TypedDict):
    """
    Configuration associated with `SELECTION`-type custom attribute definitions.
    """

    max_allowed_selections: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The maximum number of selections that can be set. The maximum value for this
    attribute is 100. The default value is 1. The value can be modified, but changing the value will not
    affect existing custom attribute values on objects. Clients need to
    handle custom attributes with more selected values than allowed by this limit.
    """

    allowed_selections: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[CatalogCustomAttributeDefinitionSelectionConfigCustomAttributeSelectionParams]]
    ]
    """
    The set of valid `CatalogCustomAttributeSelections`. Up to a maximum of 100
    selections can be defined. Can be modified.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .data_collection_options_input_type import DataCollectionOptionsInputType
import typing
from .collected_data import CollectedData
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DataCollectionOptions(UncheckedBaseModel):
    title: str = pydantic.Field()
    """
    The title text to display in the data collection flow on the Terminal.
    """

    body: str = pydantic.Field()
    """
    The body text to display under the title in the data collection screen flow on the
    Terminal.
    """

    input_type: DataCollectionOptionsInputType = pydantic.Field()
    """
    Represents the type of the input text.
    See [InputType](#type-inputtype) for possible values
    """

    collected_data: typing.Optional[CollectedData] = pydantic.Field(default=None)
    """
    The buyer’s input text from the data collection screen.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment_refund import PaymentRefundParams


class GetPaymentRefundResponseParams(typing_extensions.TypedDict):
    """
    Defines the response returned by [GetRefund](api-endpoint:Refunds-GetPaymentRefund).

    Note: If there are errors processing the request, the refund field might not be
    present or it might be present in a FAILED state.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    refund: typing_extensions.NotRequired[PaymentRefundParams]
    """
    The requested `PaymentRefund`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

V1OrderState = typing.Union[
    typing.Literal["PENDING", "OPEN", "COMPLETED", "CANCELED", "REFUNDED", "REJECTED"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogItemOptionForItemParams(typing_extensions.TypedDict):
    """
     An option that can be assigned to an item.
    For example, a t-shirt item may offer a color option or a size option.
    """

    item_option_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The unique id of the item option, used to form the dimensions of the item option matrix in a specified order.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

SubscriptionEventInfoCode = typing.Union[
    typing.Literal[
        "LOCATION_NOT_ACTIVE",
        "LOCATION_CANNOT_ACCEPT_PAYMENT",
        "CUSTOMER_DELETED",
        "CUSTOMER_NO_EMAIL",
        "CUSTOMER_NO_NAME",
        "USER_PROVIDED",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomAttributeDefinitionsClient
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.custom_attribute_definition import CustomAttributeDefinition
from ...types.list_booking_custom_attribute_definitions_response import ListBookingCustomAttributeDefinitionsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.custom_attribute_definition import CustomAttributeDefinitionParams
from ...types.create_booking_custom_attribute_definition_response import CreateBookingCustomAttributeDefinitionResponse
from ...types.retrieve_booking_custom_attribute_definition_response import (
    RetrieveBookingCustomAttributeDefinitionResponse,
)
from ...types.update_booking_custom_attribute_definition_response import UpdateBookingCustomAttributeDefinitionResponse
from ...types.delete_booking_custom_attribute_definition_response import DeleteBookingCustomAttributeDefinitionResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomAttributeDefinitionsClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomAttributeDefinition]:
        """
        Get all bookings custom attribute definitions.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.bookings.custom_attribute_definitions.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings/custom-attribute-definitions",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBookingCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListBookingCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBookingCustomAttributeDefinitionResponse:
        """
        Creates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create, with the following fields:

            - `key`

            - `name`. If provided, `name` must be unique (case-sensitive) across all visible booking-related custom attribute
            definitions for the seller.

            - `description`

            - `visibility`. Note that all custom attributes are visible in exported booking data, including those set to
            `VISIBILITY_HIDDEN`.

            - `schema`. With the exception of the `Selection` data type, the `schema` is specified as a
            simple URL to the JSON schema definition hosted on the Square CDN. For more information, see
            [Specifying the schema](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#specify-schema).

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attribute_definitions.create(
            custom_attribute_definition={},
        )
        """
        response = self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveBookingCustomAttributeDefinitionResponse:
        """
        Retrieves a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attribute_definitions.get(
            key="key",
        )
        """
        response = self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBookingCustomAttributeDefinitionResponse:
        """
        Updates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attribute_definitions.update(
            key="key",
            custom_attribute_definition={},
        )
        """
        response = self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteBookingCustomAttributeDefinitionResponse:
        """
        Deletes a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.bookings.custom_attribute_definitions.delete(
            key="key",
        )
        """
        response = self._raw_client.delete(key, request_options=request_options)
        return response.data


class AsyncCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomAttributeDefinition]:
        """
        Get all bookings custom attribute definitions.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.bookings.custom_attribute_definitions.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/bookings/custom-attribute-definitions",
            method="GET",
            params={
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListBookingCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListBookingCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    limit=limit,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBookingCustomAttributeDefinitionResponse:
        """
        Creates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create, with the following fields:

            - `key`

            - `name`. If provided, `name` must be unique (case-sensitive) across all visible booking-related custom attribute
            definitions for the seller.

            - `description`

            - `visibility`. Note that all custom attributes are visible in exported booking data, including those set to
            `VISIBILITY_HIDDEN`.

            - `schema`. With the exception of the `Selection` data type, the `schema` is specified as a
            simple URL to the JSON schema definition hosted on the Square CDN. For more information, see
            [Specifying the schema](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#specify-schema).

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attribute_definitions.create(
                custom_attribute_definition={},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveBookingCustomAttributeDefinitionResponse:
        """
        Retrieves a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attribute_definitions.get(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    async def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBookingCustomAttributeDefinitionResponse:
        """
        Updates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attribute_definitions.update(
                key="key",
                custom_attribute_definition={},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteBookingCustomAttributeDefinitionResponse:
        """
        Deletes a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteBookingCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.bookings.custom_attribute_definitions.delete(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(key, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.timecard_sort_field import TimecardSortField
from ..types.sort_order import SortOrder


class TimecardSortParams(typing_extensions.TypedDict):
    """
    Sets the sort order of search results.
    """

    field: typing_extensions.NotRequired[TimecardSortField]
    """
    The field to sort on.
    See [TimecardSortField](#type-timecardsortfield) for possible values
    """

    order: typing_extensions.NotRequired[SortOrder]
    """
    The order in which results are returned. Defaults to DESC.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .subscription_status import SubscriptionStatus
from .money import Money
from .subscription_source import SubscriptionSource
from .subscription_action import SubscriptionAction
from .phase import Phase
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Subscription(UncheckedBaseModel):
    """
    Represents a subscription purchased by a customer.

    For more information, see
    [Manage Subscriptions](https://developer.squareup.com/docs/subscriptions-api/manage-subscriptions).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the subscription.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the location associated with the subscription.
    """

    plan_variation_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the subscribed-to [subscription plan variation](entity:CatalogSubscriptionPlanVariation).
    """

    customer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the subscribing [customer](entity:Customer) profile.
    """

    start_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `YYYY-MM-DD`-formatted date (for example, 2013-01-15) to start the subscription.
    """

    canceled_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `YYYY-MM-DD`-formatted date (for example, 2013-01-15) to cancel the subscription, 
    when the subscription status changes to `CANCELED` and the subscription billing stops.
    
    If this field is not set, the subscription ends according its subscription plan.
    
    This field cannot be updated, other than being cleared.
    """

    charged_through_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `YYYY-MM-DD`-formatted date up to when the subscriber is invoiced for the
    subscription.
    
    After the invoice is sent for a given billing period,
    this date will be the last day of the billing period.
    For example,
    suppose for the month of May a subscriber gets an invoice
    (or charged the card) on May 1. For the monthly billing scenario,
    this date is then set to May 31.
    """

    status: typing.Optional[SubscriptionStatus] = pydantic.Field(default=None)
    """
    The current status of the subscription.
    See [SubscriptionStatus](#type-subscriptionstatus) for possible values
    """

    tax_percentage: typing.Optional[str] = pydantic.Field(default=None)
    """
    The tax amount applied when billing the subscription. The
    percentage is expressed in decimal form, using a `'.'` as the decimal
    separator and without a `'%'` sign. For example, a value of `7.5`
    corresponds to 7.5%.
    """

    invoice_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of the [invoices](entity:Invoice) created for the
    subscription, listed in order when the invoices were created
    (newest invoices appear first).
    """

    price_override_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    A custom price which overrides the cost of a subscription plan variation with `STATIC` pricing.
    This field does not affect itemized subscriptions with `RELATIVE` pricing. Instead, 
    you should edit the Subscription's [order template](https://developer.squareup.com/docs/subscriptions-api/manage-subscriptions#phases-and-order-templates).
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the object. When updating an object, the version
    supplied must match the version in the database, otherwise the write will
    be rejected as conflicting.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the subscription was created, in RFC 3339 format.
    """

    card_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [subscriber's](entity:Customer) [card](entity:Card)
    used to charge for the subscription.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Timezone that will be used in date calculations for the subscription.
    Defaults to the timezone of the location based on `location_id`.
    Format: the IANA Timezone Database identifier for the location timezone (for example, `America/Los_Angeles`).
    """

    source: typing.Optional[SubscriptionSource] = pydantic.Field(default=None)
    """
    The origination details of the subscription.
    """

    actions: typing.Optional[typing.List[SubscriptionAction]] = pydantic.Field(default=None)
    """
    The list of scheduled actions on this subscription. It is set only in the response from  
    [RetrieveSubscription](api-endpoint:Subscriptions-RetrieveSubscription) with the query parameter
    of `include=actions` or from 
    [SearchSubscriptions](api-endpoint:Subscriptions-SearchSubscriptions) with the input parameter 
    of `include:["actions"]`.
    """

    monthly_billing_anchor_date: typing.Optional[int] = pydantic.Field(default=None)
    """
    The day of the month on which the subscription will issue invoices and publish orders.
    """

    phases: typing.Optional[typing.List[Phase]] = pydantic.Field(default=None)
    """
    array of phases for this subscription
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemOptionValue(UncheckedBaseModel):
    """
    An enumerated value that can link a
    `CatalogItemVariation` to an item option as one of
    its item option values.
    """

    item_option_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique ID of the associated item option.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of this item option value. This is a searchable attribute for use in applicable query filters.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable description for the option value. This is a searchable attribute for use in applicable query filters.
    """

    color: typing.Optional[str] = pydantic.Field(default=None)
    """
    The HTML-supported hex color for the item option (e.g., "#ff8d4e85").
    Only displayed if `show_colors` is enabled on the parent `ItemOption`. When
    left unset, `color` defaults to white ("#ffffff") when `show_colors` is
    enabled on the parent `ItemOption`.
    """

    ordinal: typing.Optional[int] = pydantic.Field(default=None)
    """
    Determines where this option value appears in a list of option values.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .money import Money
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GiftCardActivityUnlinkedActivityRefund(UncheckedBaseModel):
    """
    Represents details about an `UNLINKED_ACTIVITY_REFUND` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: Money = pydantic.Field()
    """
    The amount added to the gift card for the refund. This value is a positive integer.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A client-specified ID that associates the gift card activity with an entity in another system.
    """

    payment_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the refunded payment. This field is not used starting in Square version 2022-06-16.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class WebhookSubscription(UncheckedBaseModel):
    """
    Represents the details of a webhook subscription, including notification URL,
    event types, and signature key.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A Square-generated unique ID for the subscription.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of this subscription.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the subscription is enabled (`true`) or not (`false`).
    """

    event_types: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The event types associated with this subscription.
    """

    notification_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL to which webhooks are sent.
    """

    api_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The API version of the subscription.
    This field is optional for `CreateWebhookSubscription`. 
    The value defaults to the API version used by the application.
    """

    signature_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-generated signature key used to validate the origin of the webhook event.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the subscription was created, in RFC 3339 format. For example, "2016-09-04T23:59:33.123Z".
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the subscription was last updated, in RFC 3339 format.
    For example, "2016-09-04T23:59:33.123Z".
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .custom_attribute import CustomAttribute
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListCustomerCustomAttributesResponse(UncheckedBaseModel):
    """
    Represents a [ListCustomerCustomAttributes](api-endpoint:CustomerCustomAttributes-ListCustomerCustomAttributes) response.
    Either `custom_attributes`, an empty object, or `errors` is present in the response. If additional
    results are available, the `cursor` field is also present along with `custom_attributes`.
    """

    custom_attributes: typing.Optional[typing.List[CustomAttribute]] = pydantic.Field(default=None)
    """
    The retrieved custom attributes. If `with_definitions` was set to `true` in the request,
    the custom attribute definition is returned in the `definition` field of each custom attribute.
    
    If no custom attributes are found, Square returns an empty object (`{}`).
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cursor to use in your next call to this endpoint to retrieve the next page of results
    for your original request. This field is present only if the request succeeded and additional
    results are available. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class OfflinePaymentDetailsParams(typing_extensions.TypedDict):
    """
    Details specific to offline payments.
    """

    client_created_at: typing_extensions.NotRequired[str]
    """
    The client-side timestamp of when the offline payment was created, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from .dispute_reason import DisputeReason
from .dispute_state import DisputeState
from .disputed_payment import DisputedPayment
from .card_brand import CardBrand
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Dispute(UncheckedBaseModel):
    """
    Represents a [dispute](https://developer.squareup.com/docs/disputes-api/overview) a cardholder initiated with their bank.
    """

    dispute_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID for this `Dispute`, generated by Square.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID for this `Dispute`, generated by Square.
    """

    amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The disputed amount, which can be less than the total transaction amount.
    For instance, if multiple items were purchased but the cardholder only initiates a dispute over some of the items.
    """

    reason: typing.Optional[DisputeReason] = pydantic.Field(default=None)
    """
    The reason why the cardholder initiated the dispute.
    See [DisputeReason](#type-disputereason) for possible values
    """

    state: typing.Optional[DisputeState] = pydantic.Field(default=None)
    """
    The current state of this dispute.
    See [DisputeState](#type-disputestate) for possible values
    """

    due_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The deadline by which the seller must respond to the dispute, in [RFC 3339 format](https://developer.squareup.com/docs/build-basics/common-data-types/working-with-dates).
    """

    disputed_payment: typing.Optional[DisputedPayment] = pydantic.Field(default=None)
    """
    The payment challenged in this dispute.
    """

    evidence_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of the evidence associated with the dispute.
    """

    card_brand: typing.Optional[CardBrand] = pydantic.Field(default=None)
    """
    The card brand used in the disputed payment.
    See [CardBrand](#type-cardbrand) for possible values
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the dispute was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the dispute was last updated, in RFC 3339 format.
    """

    brand_dispute_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the dispute in the card brand system, generated by the card brand.
    """

    reported_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the dispute was reported, in RFC 3339 format.
    """

    reported_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the dispute was reported, in RFC 3339 format.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The current version of the `Dispute`.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the location where the dispute originated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .search_orders_sort_field import SearchOrdersSortField
import pydantic
import typing
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersSort(UncheckedBaseModel):
    """
    Sorting criteria for a `SearchOrders` request. Results can only be sorted
    by a timestamp field.
    """

    sort_field: SearchOrdersSortField = pydantic.Field()
    """
    The field to sort by.
    
    __Important:__ When using a [DateTimeFilter](entity:SearchOrdersFilter),
    `sort_field` must match the timestamp field that the `DateTimeFilter` uses to
    filter. For example, if you set your `sort_field` to `CLOSED_AT` and you use a
    `DateTimeFilter`, your `DateTimeFilter` must filter for orders by their `CLOSED_AT` date.
    If this field does not match the timestamp field in `DateTimeFilter`,
    `SearchOrders` returns an error.
    
    Default: `CREATED_AT`.
    See [SearchOrdersSortField](#type-searchorderssortfield) for possible values
    """

    sort_order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    The chronological order in which results are returned. Defaults to `DESC`.
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing_extensions
import typing
from ..core.serialization import FieldMetadata
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogV1Id(UncheckedBaseModel):
    """
    A Square API V1 identifier of an item, including the object ID and its associated location ID.
    """

    catalog_v1id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="catalog_v1_id")] = (
        pydantic.Field(default=None)
    )
    """
    The ID for an object used in the Square API V1, if the object ID differs from the Square API V2 object ID.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the `Location` this Connect V1 ID is associated with.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.loyalty_promotion_trigger_limit_interval import LoyaltyPromotionTriggerLimitInterval


class LoyaltyPromotionTriggerLimitParams(typing_extensions.TypedDict):
    """
    Represents the number of times a buyer can earn points during a [loyalty promotion](entity:LoyaltyPromotion).
    If this field is not set, buyers can trigger the promotion an unlimited number of times to earn points during
    the time that the promotion is available.

    A purchase that is disqualified from earning points because of this limit might qualify for another active promotion.
    """

    times: int
    """
    The maximum number of times a buyer can trigger the promotion during the specified `interval`.
    """

    interval: typing_extensions.NotRequired[LoyaltyPromotionTriggerLimitInterval]
    """
    The time period the limit applies to.
    See [LoyaltyPromotionTriggerLimitInterval](#type-loyaltypromotiontriggerlimitinterval) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .shift_sort_field import ShiftSortField
import pydantic
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ShiftSort(UncheckedBaseModel):
    """
    Sets the sort order of search results.

    Deprecated at Square API version 2025-05-21. See the [migration notes](https://developer.squareup.com/docs/labor-api/what-it-does#migration-notes).
    """

    field: typing.Optional[ShiftSortField] = pydantic.Field(default=None)
    """
    The field to sort on.
    See [ShiftSortField](#type-shiftsortfield) for possible values
    """

    order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    The order in which results are returned. Defaults to DESC.
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class RangeParams(typing_extensions.TypedDict):
    """
    The range of a number value between the specified lower and upper bounds.
    """

    min: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The lower bound of the number range. At least one of `min` or `max` must be specified.
    If unspecified, the results will have no minimum value.
    """

    max: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The upper bound of the number range. At least one of `min` or `max` must be specified.
    If unspecified, the results will have no maximum value.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .dispute_evidence import DisputeEvidence
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListDisputeEvidenceResponse(UncheckedBaseModel):
    """
    Defines the fields in a `ListDisputeEvidence` response.
    """

    evidence: typing.Optional[typing.List[DisputeEvidence]] = pydantic.Field(default=None)
    """
    The list of evidence previously uploaded to the specified dispute.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Information about errors encountered during the request.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The pagination cursor to be used in a subsequent request.
    If unset, this is the final response. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .action_cancel_reason import ActionCancelReason
from .terminal_action_action_type import TerminalActionActionType
from .qr_code_options import QrCodeOptions
from .save_card_options import SaveCardOptions
from .signature_options import SignatureOptions
from .confirmation_options import ConfirmationOptions
from .receipt_options import ReceiptOptions
from .data_collection_options import DataCollectionOptions
from .select_options import SelectOptions
from .device_metadata import DeviceMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TerminalAction(UncheckedBaseModel):
    """
    Represents an action processed by the Square Terminal.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID for this `TerminalAction`.
    """

    device_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique Id of the device intended for this `TerminalAction`.
    The Id can be retrieved from /v2/devices api.
    """

    deadline_duration: typing.Optional[str] = pydantic.Field(default=None)
    """
    The duration as an RFC 3339 duration, after which the action will be automatically canceled.
    TerminalActions that are `PENDING` will be automatically `CANCELED` and have a cancellation reason
    of `TIMED_OUT`
    
    Default: 5 minutes from creation
    
    Maximum: 5 minutes
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of the `TerminalAction`.
    Options: `PENDING`, `IN_PROGRESS`, `CANCEL_REQUESTED`, `CANCELED`, `COMPLETED`
    """

    cancel_reason: typing.Optional[ActionCancelReason] = pydantic.Field(default=None)
    """
    The reason why `TerminalAction` is canceled. Present if the status is `CANCELED`.
    See [ActionCancelReason](#type-actioncancelreason) for possible values
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the `TerminalAction` was created as an RFC 3339 timestamp.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the `TerminalAction` was last updated as an RFC 3339 timestamp.
    """

    app_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the application that created the action.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location id the action is attached to, if a link can be made.
    """

    type: typing.Optional[TerminalActionActionType] = pydantic.Field(default=None)
    """
    Represents the type of the action.
    See [ActionType](#type-actiontype) for possible values
    """

    qr_code_options: typing.Optional[QrCodeOptions] = pydantic.Field(default=None)
    """
    Describes configuration for the QR code action. Requires `QR_CODE` type.
    """

    save_card_options: typing.Optional[SaveCardOptions] = pydantic.Field(default=None)
    """
    Describes configuration for the save-card action. Requires `SAVE_CARD` type.
    """

    signature_options: typing.Optional[SignatureOptions] = pydantic.Field(default=None)
    """
    Describes configuration for the signature capture action. Requires `SIGNATURE` type.
    """

    confirmation_options: typing.Optional[ConfirmationOptions] = pydantic.Field(default=None)
    """
    Describes configuration for the confirmation action. Requires `CONFIRMATION` type.
    """

    receipt_options: typing.Optional[ReceiptOptions] = pydantic.Field(default=None)
    """
    Describes configuration for the receipt action. Requires `RECEIPT` type.
    """

    data_collection_options: typing.Optional[DataCollectionOptions] = pydantic.Field(default=None)
    """
    Describes configuration for the data collection action. Requires `DATA_COLLECTION` type.
    """

    select_options: typing.Optional[SelectOptions] = pydantic.Field(default=None)
    """
    Describes configuration for the select action. Requires `SELECT` type.
    """

    device_metadata: typing.Optional[DeviceMetadata] = pydantic.Field(default=None)
    """
    Details about the Terminal that received the action request (such as battery level,
    operating system version, and network connection settings).
    
    Only available for `PING` action type.
    """

    await_next_action: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates the action will be linked to another action and requires a waiting dialog to be
    displayed instead of returning to the idle screen on completion of the action.
    
    Only supported on SIGNATURE, CONFIRMATION, DATA_COLLECTION, and SELECT types.
    """

    await_next_action_duration: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timeout duration of the waiting dialog as an RFC 3339 duration, after which the
    waiting dialog will no longer be displayed and the Terminal will return to the idle screen.
    
    Default: 5 minutes from when the waiting dialog is displayed
    
    Maximum: 5 minutes
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomAttributeDefinitionsClient
from ...types.visibility_filter import VisibilityFilter
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.custom_attribute_definition import CustomAttributeDefinition
from ...types.list_order_custom_attribute_definitions_response import ListOrderCustomAttributeDefinitionsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.custom_attribute_definition import CustomAttributeDefinitionParams
from ...types.create_order_custom_attribute_definition_response import CreateOrderCustomAttributeDefinitionResponse
from ...types.retrieve_order_custom_attribute_definition_response import RetrieveOrderCustomAttributeDefinitionResponse
from ...types.update_order_custom_attribute_definition_response import UpdateOrderCustomAttributeDefinitionResponse
from ...types.delete_order_custom_attribute_definition_response import DeleteOrderCustomAttributeDefinitionResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomAttributeDefinitionsClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomAttributeDefinition]:
        """
        Lists the order-related [custom attribute definitions](entity:CustomAttributeDefinition) that belong to a Square seller account.

        When all response pages are retrieved, the results include all custom attribute definitions
        that are visible to the requesting application, including those that are created by other
        applications and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that
        seller-defined custom attributes (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        visibility_filter : typing.Optional[VisibilityFilter]
            Requests that all of the custom attributes be returned, or only those that are read-only or read-write.

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.orders.custom_attribute_definitions.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/orders/custom-attribute-definitions",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrderCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListOrderCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    visibility_filter=visibility_filter,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrderCustomAttributeDefinitionResponse:
        """
        Creates an order-related custom attribute definition.  Use this endpoint to
        define a custom attribute that can be associated with orders.

        After creating a custom attribute definition, you can set the custom attribute for orders
        in the Square seller account.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Specifying the schema](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#specify-schema).
            - If provided, `name` must be unique (case-sensitive) across all visible customer-related custom attribute definitions for the seller.
            - All custom attributes are visible in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attribute_definitions.create(
            custom_attribute_definition={
                "key": "cover-count",
                "schema": {
                    "ref": "https://developer-production-s.squarecdn.com/schemas/v1/common.json#squareup.common.Number"
                },
                "name": "Cover count",
                "description": "The number of people seated at a table",
                "visibility": "VISIBILITY_READ_WRITE_VALUES",
            },
            idempotency_key="IDEMPOTENCY_KEY",
        )
        """
        response = self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveOrderCustomAttributeDefinitionResponse:
        """
        Retrieves an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attribute_definitions.get(
            key="key",
        )
        """
        response = self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrderCustomAttributeDefinitionResponse:
        """
        Updates an order-related custom attribute definition for a Square seller account.

        Only the definition owner can update a custom attribute definition. Note that sellers can view all custom attributes in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint supports sparse updates,
            so only new or changed fields need to be included in the request.  For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/orders-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attribute_definitions.update(
            key="key",
            custom_attribute_definition={
                "key": "cover-count",
                "visibility": "VISIBILITY_READ_ONLY",
                "version": 1,
            },
            idempotency_key="IDEMPOTENCY_KEY",
        )
        """
        response = self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteOrderCustomAttributeDefinitionResponse:
        """
        Deletes an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.orders.custom_attribute_definitions.delete(
            key="key",
        )
        """
        response = self._raw_client.delete(key, request_options=request_options)
        return response.data


class AsyncCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomAttributeDefinitionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomAttributeDefinitionsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomAttributeDefinition]:
        """
        Lists the order-related [custom attribute definitions](entity:CustomAttributeDefinition) that belong to a Square seller account.

        When all response pages are retrieved, the results include all custom attribute definitions
        that are visible to the requesting application, including those that are created by other
        applications and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that
        seller-defined custom attributes (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        visibility_filter : typing.Optional[VisibilityFilter]
            Requests that all of the custom attributes be returned, or only those that are read-only or read-write.

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomAttributeDefinition]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.orders.custom_attribute_definitions.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/orders/custom-attribute-definitions",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrderCustomAttributeDefinitionsResponse,
                    construct_type(
                        type_=ListOrderCustomAttributeDefinitionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    visibility_filter=visibility_filter,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attribute_definitions
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrderCustomAttributeDefinitionResponse:
        """
        Creates an order-related custom attribute definition.  Use this endpoint to
        define a custom attribute that can be associated with orders.

        After creating a custom attribute definition, you can set the custom attribute for orders
        in the Square seller account.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create. Note the following:
            - With the exception of the `Selection` data type, the `schema` is specified as a simple URL to the JSON schema
            definition hosted on the Square CDN. For more information, including supported values and constraints, see
            [Specifying the schema](https://developer.squareup.com/docs/customer-custom-attributes-api/custom-attribute-definitions#specify-schema).
            - If provided, `name` must be unique (case-sensitive) across all visible customer-related custom attribute definitions for the seller.
            - All custom attributes are visible in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attribute_definitions.create(
                custom_attribute_definition={
                    "key": "cover-count",
                    "schema": {
                        "ref": "https://developer-production-s.squarecdn.com/schemas/v1/common.json#squareup.common.Number"
                    },
                    "name": "Cover count",
                    "description": "The number of people seated at a table",
                    "visibility": "VISIBILITY_READ_WRITE_VALUES",
                },
                idempotency_key="IDEMPOTENCY_KEY",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RetrieveOrderCustomAttributeDefinitionResponse:
        """
        Retrieves an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        To retrieve a custom attribute definition created by another application, the `visibility`
        setting must be `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`. Note that seller-defined custom attributes
        (also known as custom fields) are always set to `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve.

        version : typing.Optional[int]
            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include this optional field and specify the current version of the custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attribute_definitions.get(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(key, version=version, request_options=request_options)
        return response.data

    async def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrderCustomAttributeDefinitionResponse:
        """
        Updates an order-related custom attribute definition for a Square seller account.

        Only the definition owner can update a custom attribute definition. Note that sellers can view all custom attributes in exported customer data, including those set to `VISIBILITY_HIDDEN`.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. This endpoint supports sparse updates,
            so only new or changed fields need to be included in the request.  For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/orders-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency) control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attribute_definitions.update(
                key="key",
                custom_attribute_definition={
                    "key": "cover-count",
                    "visibility": "VISIBILITY_READ_ONLY",
                    "version": 1,
                },
                idempotency_key="IDEMPOTENCY_KEY",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            key,
            custom_attribute_definition=custom_attribute_definition,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteOrderCustomAttributeDefinitionResponse:
        """
        Deletes an order-related [custom attribute definition](entity:CustomAttributeDefinition) from a Square seller account.

        Only the definition owner can delete a custom attribute definition.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteOrderCustomAttributeDefinitionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.orders.custom_attribute_definitions.delete(
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(key, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogDiscountModifyTaxBasis = typing.Union[typing.Literal["MODIFY_TAX_BASIS", "DO_NOT_MODIFY_TAX_BASIS"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TeamMemberWage(UncheckedBaseModel):
    """
    Job and wage information for a [team member](entity:TeamMember).
    This convenience object provides details needed to specify the `wage`
    field for a [timecard](entity:Timecard).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The UUID for this object.
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `TeamMember` that this wage is assigned to.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The job title that this wage relates to.
    """

    hourly_rate: typing.Optional[Money] = pydantic.Field(default=None)
    """
    Can be a custom-set hourly wage or the calculated effective hourly
    wage based on the annual wage and hours worked per week.
    """

    job_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An identifier for the [job](entity:Job) that this wage relates to.
    """

    tip_eligible: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether team members are eligible for tips when working this job.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.money import MoneyParams
from ..requests.subscription_source import SubscriptionSourceParams
from ..requests.phase import PhaseParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.create_subscription_response import CreateSubscriptionResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.bulk_swap_plan_response import BulkSwapPlanResponse
from ..requests.search_subscriptions_query import SearchSubscriptionsQueryParams
from ..types.search_subscriptions_response import SearchSubscriptionsResponse
from ..types.get_subscription_response import GetSubscriptionResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..requests.subscription import SubscriptionParams
from ..types.update_subscription_response import UpdateSubscriptionResponse
from ..types.delete_subscription_action_response import DeleteSubscriptionActionResponse
from ..types.change_billing_anchor_date_response import ChangeBillingAnchorDateResponse
from ..types.cancel_subscription_response import CancelSubscriptionResponse
from ..types.change_timing import ChangeTiming
from ..types.pause_subscription_response import PauseSubscriptionResponse
from ..types.resume_subscription_response import ResumeSubscriptionResponse
from ..requests.phase_input import PhaseInputParams
from ..types.swap_plan_response import SwapPlanResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        location_id: str,
        customer_id: str,
        idempotency_key: typing.Optional[str] = OMIT,
        plan_variation_id: typing.Optional[str] = OMIT,
        start_date: typing.Optional[str] = OMIT,
        canceled_date: typing.Optional[str] = OMIT,
        tax_percentage: typing.Optional[str] = OMIT,
        price_override_money: typing.Optional[MoneyParams] = OMIT,
        card_id: typing.Optional[str] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        source: typing.Optional[SubscriptionSourceParams] = OMIT,
        monthly_billing_anchor_date: typing.Optional[int] = OMIT,
        phases: typing.Optional[typing.Sequence[PhaseParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateSubscriptionResponse]:
        """
        Enrolls a customer in a subscription.

        If you provide a card on file in the request, Square charges the card for
        the subscription. Otherwise, Square sends an invoice to the customer's email
        address. The subscription starts immediately, unless the request includes
        the optional `start_date`. Each individual subscription is associated with a particular location.

        For more information, see [Create a subscription](https://developer.squareup.com/docs/subscriptions-api/manage-subscriptions#create-a-subscription).

        Parameters
        ----------
        location_id : str
            The ID of the location the subscription is associated with.

        customer_id : str
            The ID of the [customer](entity:Customer) subscribing to the subscription plan variation.

        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreateSubscription` request.
            If you do not provide a unique string (or provide an empty string as the value),
            the endpoint treats each request as independent.

            For more information, see [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        plan_variation_id : typing.Optional[str]
            The ID of the [subscription plan variation](https://developer.squareup.com/docs/subscriptions-api/plans-and-variations#plan-variations) created using the Catalog API.

        start_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date to start the subscription.
            If it is unspecified, the subscription starts immediately.

        canceled_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the newly created subscription is scheduled for cancellation.

            This date overrides the cancellation date set in the plan variation configuration.
            If the cancellation date is earlier than the end date of a subscription cycle, the subscription stops
            at the canceled date and the subscriber is sent a prorated invoice at the beginning of the canceled cycle.

            When the subscription plan of the newly created subscription has a fixed number of cycles and the `canceled_date`
            occurs before the subscription plan expires, the specified `canceled_date` sets the date when the subscription
            stops through the end of the last cycle.

        tax_percentage : typing.Optional[str]
            The tax to add when billing the subscription.
            The percentage is expressed in decimal form, using a `'.'` as the decimal
            separator and without a `'%'` sign. For example, a value of 7.5
            corresponds to 7.5%.

        price_override_money : typing.Optional[MoneyParams]
            A custom price which overrides the cost of a subscription plan variation with `STATIC` pricing.
            This field does not affect itemized subscriptions with `RELATIVE` pricing. Instead,
            you should edit the Subscription's [order template](https://developer.squareup.com/docs/subscriptions-api/manage-subscriptions#phases-and-order-templates).

        card_id : typing.Optional[str]
            The ID of the [subscriber's](entity:Customer) [card](entity:Card) to charge.
            If it is not specified, the subscriber receives an invoice via email with a link to pay for their subscription.

        timezone : typing.Optional[str]
            The timezone that is used in date calculations for the subscription. If unset, defaults to
            the location timezone. If a timezone is not configured for the location, defaults to "America/New_York".
            Format: the IANA Timezone Database identifier for the location timezone. For
            a list of time zones, see [List of tz database time zones](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones).

        source : typing.Optional[SubscriptionSourceParams]
            The origination details of the subscription.

        monthly_billing_anchor_date : typing.Optional[int]
            The day-of-the-month to change the billing date to.

        phases : typing.Optional[typing.Sequence[PhaseParams]]
            array of phases for this subscription

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/subscriptions",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "location_id": location_id,
                "plan_variation_id": plan_variation_id,
                "customer_id": customer_id,
                "start_date": start_date,
                "canceled_date": canceled_date,
                "tax_percentage": tax_percentage,
                "price_override_money": convert_and_respect_annotation_metadata(
                    object_=price_override_money, annotation=MoneyParams, direction="write"
                ),
                "card_id": card_id,
                "timezone": timezone,
                "source": convert_and_respect_annotation_metadata(
                    object_=source, annotation=SubscriptionSourceParams, direction="write"
                ),
                "monthly_billing_anchor_date": monthly_billing_anchor_date,
                "phases": convert_and_respect_annotation_metadata(
                    object_=phases, annotation=typing.Sequence[PhaseParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSubscriptionResponse,
                    construct_type(
                        type_=CreateSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_swap_plan(
        self,
        *,
        new_plan_variation_id: str,
        old_plan_variation_id: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BulkSwapPlanResponse]:
        """
        Schedules a plan variation change for all active subscriptions under a given plan
        variation. For more information, see [Swap Subscription Plan Variations](https://developer.squareup.com/docs/subscriptions-api/swap-plan-variations).

        Parameters
        ----------
        new_plan_variation_id : str
            The ID of the new subscription plan variation.

            This field is required.

        old_plan_variation_id : str
            The ID of the plan variation whose subscriptions should be swapped. Active subscriptions
            using this plan variation will be subscribed to the new plan variation on their next billing
            day.

        location_id : str
            The ID of the location to associate with the swapped subscriptions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkSwapPlanResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/subscriptions/bulk-swap-plan",
            method="POST",
            json={
                "new_plan_variation_id": new_plan_variation_id,
                "old_plan_variation_id": old_plan_variation_id,
                "location_id": location_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkSwapPlanResponse,
                    construct_type(
                        type_=BulkSwapPlanResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[SearchSubscriptionsQueryParams] = OMIT,
        include: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchSubscriptionsResponse]:
        """
        Searches for subscriptions.

        Results are ordered chronologically by subscription creation date. If
        the request specifies more than one location ID,
        the endpoint orders the result
        by location ID, and then by creation date within each location. If no locations are given
        in the query, all locations are searched.

        You can also optionally specify `customer_ids` to search by customer.
        If left unset, all customers
        associated with the specified locations are returned.
        If the request specifies customer IDs, the endpoint orders results
        first by location, within location by customer ID, and within
        customer by subscription creation date.

        Parameters
        ----------
        cursor : typing.Optional[str]
            When the total number of resulting subscriptions exceeds the limit of a paged response,
            specify the cursor returned from a preceding response here to fetch the next set of results.
            If the cursor is unset, the response contains the last page of the results.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The upper limit on the number of subscriptions to return
            in a paged response.

        query : typing.Optional[SearchSubscriptionsQueryParams]
            A subscription query consisting of specified filtering conditions.

            If this `query` field is unspecified, the `SearchSubscriptions` call will return all subscriptions.

        include : typing.Optional[typing.Sequence[str]]
            An option to include related information in the response.

            The supported values are:

            - `actions`: to include scheduled actions on the targeted subscriptions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchSubscriptionsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/subscriptions/search",
            method="POST",
            json={
                "cursor": cursor,
                "limit": limit,
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchSubscriptionsQueryParams, direction="write"
                ),
                "include": include,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchSubscriptionsResponse,
                    construct_type(
                        type_=SearchSubscriptionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        subscription_id: str,
        *,
        include: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetSubscriptionResponse]:
        """
        Retrieves a specific subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to retrieve.

        include : typing.Optional[str]
            A query parameter to specify related information to be included in the response.

            The supported query parameter values are:

            - `actions`: to include scheduled actions on the targeted subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            params={
                "include": include,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSubscriptionResponse,
                    construct_type(
                        type_=GetSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        subscription_id: str,
        *,
        subscription: typing.Optional[SubscriptionParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateSubscriptionResponse]:
        """
        Updates a subscription by modifying or clearing `subscription` field values.
        To clear a field, set its value to `null`.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to update.

        subscription : typing.Optional[SubscriptionParams]
            The subscription object containing the current version, and fields to update.
            Unset fields will be left at their current server values, and JSON `null` values will
            be treated as a request to clear the relevant data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}",
            method="PUT",
            json={
                "subscription": convert_and_respect_annotation_metadata(
                    object_=subscription, annotation=SubscriptionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSubscriptionResponse,
                    construct_type(
                        type_=UpdateSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_action(
        self, subscription_id: str, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteSubscriptionActionResponse]:
        """
        Deletes a scheduled action for a subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription the targeted action is to act upon.

        action_id : str
            The ID of the targeted action to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteSubscriptionActionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/actions/{jsonable_encoder(action_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteSubscriptionActionResponse,
                    construct_type(
                        type_=DeleteSubscriptionActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def change_billing_anchor_date(
        self,
        subscription_id: str,
        *,
        monthly_billing_anchor_date: typing.Optional[int] = OMIT,
        effective_date: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ChangeBillingAnchorDateResponse]:
        """
        Changes the [billing anchor date](https://developer.squareup.com/docs/subscriptions-api/subscription-billing#billing-dates)
        for a subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to update the billing anchor date.

        monthly_billing_anchor_date : typing.Optional[int]
            The anchor day for the billing cycle.

        effective_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the scheduled `BILLING_ANCHOR_CHANGE` action takes
            place on the subscription.

            When this date is unspecified or falls within the current billing cycle, the billing anchor date
            is changed immediately.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChangeBillingAnchorDateResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/billing-anchor",
            method="POST",
            json={
                "monthly_billing_anchor_date": monthly_billing_anchor_date,
                "effective_date": effective_date,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangeBillingAnchorDateResponse,
                    construct_type(
                        type_=ChangeBillingAnchorDateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CancelSubscriptionResponse]:
        """
        Schedules a `CANCEL` action to cancel an active subscription. This
        sets the `canceled_date` field to the end of the active billing period. After this date,
        the subscription status changes from ACTIVE to CANCELED.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelSubscriptionResponse,
                    construct_type(
                        type_=CancelSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def pause(
        self,
        subscription_id: str,
        *,
        pause_effective_date: typing.Optional[str] = OMIT,
        pause_cycle_duration: typing.Optional[int] = OMIT,
        resume_effective_date: typing.Optional[str] = OMIT,
        resume_change_timing: typing.Optional[ChangeTiming] = OMIT,
        pause_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PauseSubscriptionResponse]:
        """
        Schedules a `PAUSE` action to pause an active subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to pause.

        pause_effective_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the scheduled `PAUSE` action takes place on the subscription.

            When this date is unspecified or falls within the current billing cycle, the subscription is paused
            on the starting date of the next billing cycle.

        pause_cycle_duration : typing.Optional[int]
            The number of billing cycles the subscription will be paused before it is reactivated.

            When this is set, a `RESUME` action is also scheduled to take place on the subscription at
            the end of the specified pause cycle duration. In this case, neither `resume_effective_date`
            nor `resume_change_timing` may be specified.

        resume_effective_date : typing.Optional[str]
            The date when the subscription is reactivated by a scheduled `RESUME` action.
            This date must be at least one billing cycle ahead of `pause_effective_date`.

        resume_change_timing : typing.Optional[ChangeTiming]
            The timing whether the subscription is reactivated immediately or at the end of the billing cycle, relative to
            `resume_effective_date`.
            See [ChangeTiming](#type-changetiming) for possible values

        pause_reason : typing.Optional[str]
            The user-provided reason to pause the subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PauseSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/pause",
            method="POST",
            json={
                "pause_effective_date": pause_effective_date,
                "pause_cycle_duration": pause_cycle_duration,
                "resume_effective_date": resume_effective_date,
                "resume_change_timing": resume_change_timing,
                "pause_reason": pause_reason,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PauseSubscriptionResponse,
                    construct_type(
                        type_=PauseSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def resume(
        self,
        subscription_id: str,
        *,
        resume_effective_date: typing.Optional[str] = OMIT,
        resume_change_timing: typing.Optional[ChangeTiming] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ResumeSubscriptionResponse]:
        """
        Schedules a `RESUME` action to resume a paused or a deactivated subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to resume.

        resume_effective_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the subscription reactivated.

        resume_change_timing : typing.Optional[ChangeTiming]
            The timing to resume a subscription, relative to the specified
            `resume_effective_date` attribute value.
            See [ChangeTiming](#type-changetiming) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResumeSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/resume",
            method="POST",
            json={
                "resume_effective_date": resume_effective_date,
                "resume_change_timing": resume_change_timing,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResumeSubscriptionResponse,
                    construct_type(
                        type_=ResumeSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def swap_plan(
        self,
        subscription_id: str,
        *,
        new_plan_variation_id: typing.Optional[str] = OMIT,
        phases: typing.Optional[typing.Sequence[PhaseInputParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SwapPlanResponse]:
        """
        Schedules a `SWAP_PLAN` action to swap a subscription plan variation in an existing subscription.
        For more information, see [Swap Subscription Plan Variations](https://developer.squareup.com/docs/subscriptions-api/swap-plan-variations).

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to swap the subscription plan for.

        new_plan_variation_id : typing.Optional[str]
            The ID of the new subscription plan variation.

            This field is required.

        phases : typing.Optional[typing.Sequence[PhaseInputParams]]
            A list of PhaseInputs, to pass phase-specific information used in the swap.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SwapPlanResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/swap-plan",
            method="POST",
            json={
                "new_plan_variation_id": new_plan_variation_id,
                "phases": convert_and_respect_annotation_metadata(
                    object_=phases, annotation=typing.Optional[typing.Sequence[PhaseInputParams]], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SwapPlanResponse,
                    construct_type(
                        type_=SwapPlanResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        location_id: str,
        customer_id: str,
        idempotency_key: typing.Optional[str] = OMIT,
        plan_variation_id: typing.Optional[str] = OMIT,
        start_date: typing.Optional[str] = OMIT,
        canceled_date: typing.Optional[str] = OMIT,
        tax_percentage: typing.Optional[str] = OMIT,
        price_override_money: typing.Optional[MoneyParams] = OMIT,
        card_id: typing.Optional[str] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        source: typing.Optional[SubscriptionSourceParams] = OMIT,
        monthly_billing_anchor_date: typing.Optional[int] = OMIT,
        phases: typing.Optional[typing.Sequence[PhaseParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateSubscriptionResponse]:
        """
        Enrolls a customer in a subscription.

        If you provide a card on file in the request, Square charges the card for
        the subscription. Otherwise, Square sends an invoice to the customer's email
        address. The subscription starts immediately, unless the request includes
        the optional `start_date`. Each individual subscription is associated with a particular location.

        For more information, see [Create a subscription](https://developer.squareup.com/docs/subscriptions-api/manage-subscriptions#create-a-subscription).

        Parameters
        ----------
        location_id : str
            The ID of the location the subscription is associated with.

        customer_id : str
            The ID of the [customer](entity:Customer) subscribing to the subscription plan variation.

        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreateSubscription` request.
            If you do not provide a unique string (or provide an empty string as the value),
            the endpoint treats each request as independent.

            For more information, see [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        plan_variation_id : typing.Optional[str]
            The ID of the [subscription plan variation](https://developer.squareup.com/docs/subscriptions-api/plans-and-variations#plan-variations) created using the Catalog API.

        start_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date to start the subscription.
            If it is unspecified, the subscription starts immediately.

        canceled_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the newly created subscription is scheduled for cancellation.

            This date overrides the cancellation date set in the plan variation configuration.
            If the cancellation date is earlier than the end date of a subscription cycle, the subscription stops
            at the canceled date and the subscriber is sent a prorated invoice at the beginning of the canceled cycle.

            When the subscription plan of the newly created subscription has a fixed number of cycles and the `canceled_date`
            occurs before the subscription plan expires, the specified `canceled_date` sets the date when the subscription
            stops through the end of the last cycle.

        tax_percentage : typing.Optional[str]
            The tax to add when billing the subscription.
            The percentage is expressed in decimal form, using a `'.'` as the decimal
            separator and without a `'%'` sign. For example, a value of 7.5
            corresponds to 7.5%.

        price_override_money : typing.Optional[MoneyParams]
            A custom price which overrides the cost of a subscription plan variation with `STATIC` pricing.
            This field does not affect itemized subscriptions with `RELATIVE` pricing. Instead,
            you should edit the Subscription's [order template](https://developer.squareup.com/docs/subscriptions-api/manage-subscriptions#phases-and-order-templates).

        card_id : typing.Optional[str]
            The ID of the [subscriber's](entity:Customer) [card](entity:Card) to charge.
            If it is not specified, the subscriber receives an invoice via email with a link to pay for their subscription.

        timezone : typing.Optional[str]
            The timezone that is used in date calculations for the subscription. If unset, defaults to
            the location timezone. If a timezone is not configured for the location, defaults to "America/New_York".
            Format: the IANA Timezone Database identifier for the location timezone. For
            a list of time zones, see [List of tz database time zones](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones).

        source : typing.Optional[SubscriptionSourceParams]
            The origination details of the subscription.

        monthly_billing_anchor_date : typing.Optional[int]
            The day-of-the-month to change the billing date to.

        phases : typing.Optional[typing.Sequence[PhaseParams]]
            array of phases for this subscription

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/subscriptions",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "location_id": location_id,
                "plan_variation_id": plan_variation_id,
                "customer_id": customer_id,
                "start_date": start_date,
                "canceled_date": canceled_date,
                "tax_percentage": tax_percentage,
                "price_override_money": convert_and_respect_annotation_metadata(
                    object_=price_override_money, annotation=MoneyParams, direction="write"
                ),
                "card_id": card_id,
                "timezone": timezone,
                "source": convert_and_respect_annotation_metadata(
                    object_=source, annotation=SubscriptionSourceParams, direction="write"
                ),
                "monthly_billing_anchor_date": monthly_billing_anchor_date,
                "phases": convert_and_respect_annotation_metadata(
                    object_=phases, annotation=typing.Sequence[PhaseParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSubscriptionResponse,
                    construct_type(
                        type_=CreateSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_swap_plan(
        self,
        *,
        new_plan_variation_id: str,
        old_plan_variation_id: str,
        location_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BulkSwapPlanResponse]:
        """
        Schedules a plan variation change for all active subscriptions under a given plan
        variation. For more information, see [Swap Subscription Plan Variations](https://developer.squareup.com/docs/subscriptions-api/swap-plan-variations).

        Parameters
        ----------
        new_plan_variation_id : str
            The ID of the new subscription plan variation.

            This field is required.

        old_plan_variation_id : str
            The ID of the plan variation whose subscriptions should be swapped. Active subscriptions
            using this plan variation will be subscribed to the new plan variation on their next billing
            day.

        location_id : str
            The ID of the location to associate with the swapped subscriptions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkSwapPlanResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/subscriptions/bulk-swap-plan",
            method="POST",
            json={
                "new_plan_variation_id": new_plan_variation_id,
                "old_plan_variation_id": old_plan_variation_id,
                "location_id": location_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkSwapPlanResponse,
                    construct_type(
                        type_=BulkSwapPlanResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[SearchSubscriptionsQueryParams] = OMIT,
        include: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchSubscriptionsResponse]:
        """
        Searches for subscriptions.

        Results are ordered chronologically by subscription creation date. If
        the request specifies more than one location ID,
        the endpoint orders the result
        by location ID, and then by creation date within each location. If no locations are given
        in the query, all locations are searched.

        You can also optionally specify `customer_ids` to search by customer.
        If left unset, all customers
        associated with the specified locations are returned.
        If the request specifies customer IDs, the endpoint orders results
        first by location, within location by customer ID, and within
        customer by subscription creation date.

        Parameters
        ----------
        cursor : typing.Optional[str]
            When the total number of resulting subscriptions exceeds the limit of a paged response,
            specify the cursor returned from a preceding response here to fetch the next set of results.
            If the cursor is unset, the response contains the last page of the results.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The upper limit on the number of subscriptions to return
            in a paged response.

        query : typing.Optional[SearchSubscriptionsQueryParams]
            A subscription query consisting of specified filtering conditions.

            If this `query` field is unspecified, the `SearchSubscriptions` call will return all subscriptions.

        include : typing.Optional[typing.Sequence[str]]
            An option to include related information in the response.

            The supported values are:

            - `actions`: to include scheduled actions on the targeted subscriptions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchSubscriptionsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/subscriptions/search",
            method="POST",
            json={
                "cursor": cursor,
                "limit": limit,
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchSubscriptionsQueryParams, direction="write"
                ),
                "include": include,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchSubscriptionsResponse,
                    construct_type(
                        type_=SearchSubscriptionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        subscription_id: str,
        *,
        include: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetSubscriptionResponse]:
        """
        Retrieves a specific subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to retrieve.

        include : typing.Optional[str]
            A query parameter to specify related information to be included in the response.

            The supported query parameter values are:

            - `actions`: to include scheduled actions on the targeted subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            params={
                "include": include,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSubscriptionResponse,
                    construct_type(
                        type_=GetSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        subscription_id: str,
        *,
        subscription: typing.Optional[SubscriptionParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateSubscriptionResponse]:
        """
        Updates a subscription by modifying or clearing `subscription` field values.
        To clear a field, set its value to `null`.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to update.

        subscription : typing.Optional[SubscriptionParams]
            The subscription object containing the current version, and fields to update.
            Unset fields will be left at their current server values, and JSON `null` values will
            be treated as a request to clear the relevant data.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}",
            method="PUT",
            json={
                "subscription": convert_and_respect_annotation_metadata(
                    object_=subscription, annotation=SubscriptionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSubscriptionResponse,
                    construct_type(
                        type_=UpdateSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_action(
        self, subscription_id: str, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteSubscriptionActionResponse]:
        """
        Deletes a scheduled action for a subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription the targeted action is to act upon.

        action_id : str
            The ID of the targeted action to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteSubscriptionActionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/actions/{jsonable_encoder(action_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteSubscriptionActionResponse,
                    construct_type(
                        type_=DeleteSubscriptionActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def change_billing_anchor_date(
        self,
        subscription_id: str,
        *,
        monthly_billing_anchor_date: typing.Optional[int] = OMIT,
        effective_date: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ChangeBillingAnchorDateResponse]:
        """
        Changes the [billing anchor date](https://developer.squareup.com/docs/subscriptions-api/subscription-billing#billing-dates)
        for a subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to update the billing anchor date.

        monthly_billing_anchor_date : typing.Optional[int]
            The anchor day for the billing cycle.

        effective_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the scheduled `BILLING_ANCHOR_CHANGE` action takes
            place on the subscription.

            When this date is unspecified or falls within the current billing cycle, the billing anchor date
            is changed immediately.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChangeBillingAnchorDateResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/billing-anchor",
            method="POST",
            json={
                "monthly_billing_anchor_date": monthly_billing_anchor_date,
                "effective_date": effective_date,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangeBillingAnchorDateResponse,
                    construct_type(
                        type_=ChangeBillingAnchorDateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CancelSubscriptionResponse]:
        """
        Schedules a `CANCEL` action to cancel an active subscription. This
        sets the `canceled_date` field to the end of the active billing period. After this date,
        the subscription status changes from ACTIVE to CANCELED.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelSubscriptionResponse,
                    construct_type(
                        type_=CancelSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def pause(
        self,
        subscription_id: str,
        *,
        pause_effective_date: typing.Optional[str] = OMIT,
        pause_cycle_duration: typing.Optional[int] = OMIT,
        resume_effective_date: typing.Optional[str] = OMIT,
        resume_change_timing: typing.Optional[ChangeTiming] = OMIT,
        pause_reason: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PauseSubscriptionResponse]:
        """
        Schedules a `PAUSE` action to pause an active subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to pause.

        pause_effective_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the scheduled `PAUSE` action takes place on the subscription.

            When this date is unspecified or falls within the current billing cycle, the subscription is paused
            on the starting date of the next billing cycle.

        pause_cycle_duration : typing.Optional[int]
            The number of billing cycles the subscription will be paused before it is reactivated.

            When this is set, a `RESUME` action is also scheduled to take place on the subscription at
            the end of the specified pause cycle duration. In this case, neither `resume_effective_date`
            nor `resume_change_timing` may be specified.

        resume_effective_date : typing.Optional[str]
            The date when the subscription is reactivated by a scheduled `RESUME` action.
            This date must be at least one billing cycle ahead of `pause_effective_date`.

        resume_change_timing : typing.Optional[ChangeTiming]
            The timing whether the subscription is reactivated immediately or at the end of the billing cycle, relative to
            `resume_effective_date`.
            See [ChangeTiming](#type-changetiming) for possible values

        pause_reason : typing.Optional[str]
            The user-provided reason to pause the subscription.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PauseSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/pause",
            method="POST",
            json={
                "pause_effective_date": pause_effective_date,
                "pause_cycle_duration": pause_cycle_duration,
                "resume_effective_date": resume_effective_date,
                "resume_change_timing": resume_change_timing,
                "pause_reason": pause_reason,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PauseSubscriptionResponse,
                    construct_type(
                        type_=PauseSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def resume(
        self,
        subscription_id: str,
        *,
        resume_effective_date: typing.Optional[str] = OMIT,
        resume_change_timing: typing.Optional[ChangeTiming] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ResumeSubscriptionResponse]:
        """
        Schedules a `RESUME` action to resume a paused or a deactivated subscription.

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to resume.

        resume_effective_date : typing.Optional[str]
            The `YYYY-MM-DD`-formatted date when the subscription reactivated.

        resume_change_timing : typing.Optional[ChangeTiming]
            The timing to resume a subscription, relative to the specified
            `resume_effective_date` attribute value.
            See [ChangeTiming](#type-changetiming) for possible values

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResumeSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/resume",
            method="POST",
            json={
                "resume_effective_date": resume_effective_date,
                "resume_change_timing": resume_change_timing,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResumeSubscriptionResponse,
                    construct_type(
                        type_=ResumeSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def swap_plan(
        self,
        subscription_id: str,
        *,
        new_plan_variation_id: typing.Optional[str] = OMIT,
        phases: typing.Optional[typing.Sequence[PhaseInputParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SwapPlanResponse]:
        """
        Schedules a `SWAP_PLAN` action to swap a subscription plan variation in an existing subscription.
        For more information, see [Swap Subscription Plan Variations](https://developer.squareup.com/docs/subscriptions-api/swap-plan-variations).

        Parameters
        ----------
        subscription_id : str
            The ID of the subscription to swap the subscription plan for.

        new_plan_variation_id : typing.Optional[str]
            The ID of the new subscription plan variation.

            This field is required.

        phases : typing.Optional[typing.Sequence[PhaseInputParams]]
            A list of PhaseInputs, to pass phase-specific information used in the swap.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SwapPlanResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/subscriptions/{jsonable_encoder(subscription_id)}/swap-plan",
            method="POST",
            json={
                "new_plan_variation_id": new_plan_variation_id,
                "phases": convert_and_respect_annotation_metadata(
                    object_=phases, annotation=typing.Optional[typing.Sequence[PhaseInputParams]], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SwapPlanResponse,
                    construct_type(
                        type_=SwapPlanResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

SubscriptionEventSubscriptionEventType = typing.Union[
    typing.Literal[
        "START_SUBSCRIPTION",
        "PLAN_CHANGE",
        "STOP_SUBSCRIPTION",
        "DEACTIVATE_SUBSCRIPTION",
        "RESUME_SUBSCRIPTION",
        "PAUSE_SUBSCRIPTION",
        "BILLING_ANCHOR_DATE_CHANGED",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .scheduled_shift import ScheduledShift
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class UpdateScheduledShiftResponse(UncheckedBaseModel):
    """
    Represents an [UpdateScheduledShift](api-endpoint:Labor-UpdateScheduledShift) response.
    Either `scheduled_shift` or `errors` is present in the response.
    """

    scheduled_shift: typing.Optional[ScheduledShift] = pydantic.Field(default=None)
    """
    The updated scheduled shift. To make the changes public, call 
    [PublishScheduledShift](api-endpoint:Labor-PublishScheduledShift) or
    [BulkPublishScheduledShifts](api-endpoint:Labor-BulkPublishScheduledShifts).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .get_team_member_booking_profile_response import GetTeamMemberBookingProfileResponseParams
from .error import ErrorParams


class BulkRetrieveTeamMemberBookingProfilesResponseParams(typing_extensions.TypedDict):
    """
    Response payload for the [BulkRetrieveTeamMemberBookingProfiles](api-endpoint:Bookings-BulkRetrieveTeamMemberBookingProfiles) endpoint.
    """

    team_member_booking_profiles: typing_extensions.NotRequired[
        typing.Dict[str, GetTeamMemberBookingProfileResponseParams]
    ]
    """
    The returned team members' booking profiles, as a map with `team_member_id` as the key and [TeamMemberBookingProfile](entity:TeamMemberBookingProfile) the value.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .loyalty_promotion_trigger_limit_interval import LoyaltyPromotionTriggerLimitInterval
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyPromotionTriggerLimit(UncheckedBaseModel):
    """
    Represents the number of times a buyer can earn points during a [loyalty promotion](entity:LoyaltyPromotion).
    If this field is not set, buyers can trigger the promotion an unlimited number of times to earn points during
    the time that the promotion is available.

    A purchase that is disqualified from earning points because of this limit might qualify for another active promotion.
    """

    times: int = pydantic.Field()
    """
    The maximum number of times a buyer can trigger the promotion during the specified `interval`.
    """

    interval: typing.Optional[LoyaltyPromotionTriggerLimitInterval] = pydantic.Field(default=None)
    """
    The time period the limit applies to.
    See [LoyaltyPromotionTriggerLimitInterval](#type-loyaltypromotiontriggerlimitinterval) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .update_vendor_response import UpdateVendorResponse
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BatchUpdateVendorsResponse(UncheckedBaseModel):
    """
    Represents an output from a call to [BulkUpdateVendors](api-endpoint:Vendors-BulkUpdateVendors).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Errors encountered when the request fails.
    """

    responses: typing.Optional[typing.Dict[str, UpdateVendorResponse]] = pydantic.Field(default=None)
    """
    A set of [UpdateVendorResponse](entity:UpdateVendorResponse) objects encapsulating successfully created [Vendor](entity:Vendor)
    objects or error responses for failed attempts. The set is represented by a collection of `Vendor`-ID/`UpdateVendorResponse`-object or 
    `Vendor`-ID/error-object pairs.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class OrderEntryParams(typing_extensions.TypedDict):
    """
    A lightweight description of an [order](entity:Order) that is returned when
    `returned_entries` is `true` on a [SearchOrdersRequest](api-endpoint:Orders-SearchOrders).
    """

    order_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the order.
    """

    version: typing_extensions.NotRequired[int]
    """
    The version number, which is incremented each time an update is committed to the order.
    Orders that were not created through the API do not include a version number and
    therefore cannot be updated.
    
    [Read more about working with versions.](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders)
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The location ID the order belongs to.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.component_component_type import ComponentComponentType
import typing_extensions
from .device_component_details_application_details import DeviceComponentDetailsApplicationDetailsParams
from .device_component_details_card_reader_details import DeviceComponentDetailsCardReaderDetailsParams
from .device_component_details_battery_details import DeviceComponentDetailsBatteryDetailsParams
from .device_component_details_wi_fi_details import DeviceComponentDetailsWiFiDetailsParams
from .device_component_details_ethernet_details import DeviceComponentDetailsEthernetDetailsParams


class ComponentParams(typing_extensions.TypedDict):
    """
    The wrapper object for the component entries of a given component type.
    """

    type: ComponentComponentType
    """
    The type of this component. Each component type has expected properties expressed
    in a structured format within its corresponding `*_details` field.
    See [ComponentType](#type-componenttype) for possible values
    """

    application_details: typing_extensions.NotRequired[DeviceComponentDetailsApplicationDetailsParams]
    """
    Structured data for an `Application`, set for Components of type `APPLICATION`.
    """

    card_reader_details: typing_extensions.NotRequired[DeviceComponentDetailsCardReaderDetailsParams]
    """
    Structured data for a `CardReader`, set for Components of type `CARD_READER`.
    """

    battery_details: typing_extensions.NotRequired[DeviceComponentDetailsBatteryDetailsParams]
    """
    Structured data for a `Battery`, set for Components of type `BATTERY`.
    """

    wifi_details: typing_extensions.NotRequired[DeviceComponentDetailsWiFiDetailsParams]
    """
    Structured data for a `WiFi` interface, set for Components of type `WIFI`.
    """

    ethernet_details: typing_extensions.NotRequired[DeviceComponentDetailsEthernetDetailsParams]
    """
    Structured data for an `Ethernet` interface, set for Components of type `ETHERNET`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class LoyaltyEventAccumulatePromotionPointsParams(typing_extensions.TypedDict):
    """
    Provides metadata when the event `type` is `ACCUMULATE_PROMOTION_POINTS`.
    """

    loyalty_program_id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the [loyalty program](entity:LoyaltyProgram).
    """

    loyalty_promotion_id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the [loyalty promotion](entity:LoyaltyPromotion).
    """

    points: int
    """
    The number of points earned by the event.
    """

    order_id: str
    """
    The ID of the [order](entity:Order) for which the buyer earned the promotion points.
    Only applications that use the Orders API to process orders can trigger this event.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .loyalty_reward_status import LoyaltyRewardStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchLoyaltyRewardsRequestLoyaltyRewardQuery(UncheckedBaseModel):
    """
    The set of search requirements.
    """

    loyalty_account_id: str = pydantic.Field()
    """
    The ID of the [loyalty account](entity:LoyaltyAccount) to which the loyalty reward belongs.
    """

    status: typing.Optional[LoyaltyRewardStatus] = pydantic.Field(default=None)
    """
    The status of the loyalty reward.
    See [LoyaltyRewardStatus](#type-loyaltyrewardstatus) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .destination_details_card_refund_details import DestinationDetailsCardRefundDetails
import pydantic
from .destination_details_cash_refund_details import DestinationDetailsCashRefundDetails
from .destination_details_external_refund_details import DestinationDetailsExternalRefundDetails
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DestinationDetails(UncheckedBaseModel):
    """
    Details about a refund's destination.
    """

    card_details: typing.Optional[DestinationDetailsCardRefundDetails] = pydantic.Field(default=None)
    """
    Details about a card refund. Only populated if the destination_type is `CARD`.
    """

    cash_details: typing.Optional[DestinationDetailsCashRefundDetails] = pydantic.Field(default=None)
    """
    Details about a cash refund. Only populated if the destination_type is `CASH`.
    """

    external_details: typing.Optional[DestinationDetailsExternalRefundDetails] = pydantic.Field(default=None)
    """
    Details about an external refund. Only populated if the destination_type is `EXTERNAL`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
